@@ -109,7 +109,7 @@ init_chartab()
 		else
 		    c2 = *p++;
 	    }
-	    if (c < 0 || (c2 < c && c2 != -1) || c2 >= 256 ||
+	    if (c <= 0 || (c2 < c && c2 != -1) || c2 >= 256 ||
 						    !(*p == NUL || *p == ','))
 		return FAIL;
 
