@@ -294,6 +294,7 @@ typedef unsigned long	long_u;
 
 #define WILD_LIST_NOTFOUND	1
 #define WILD_HOME_REPLACE	2
+#define WILD_USE_NL		4	/* separate names with '\n' */
 
 /* Flags for expand_wildcards() */
 #define EW_DIR		1	/* include directory names */
@@ -450,13 +451,15 @@ typedef unsigned long	long_u;
 #define DT_LAST		6	/* jump to first match of same tag */
 #define DT_SELECT	7	/* jump to selection from list */
 #define DT_HELP		8	/* like DT_TAG, but no wildcards */
+#define DT_JUMP		9	/* jump to new tag or selection from list */
 
 /*
  * flags for find_tags().
  */
 #define TAG_HELP	1	/* only search for help tags */
 #define TAG_NAMES	2	/* only return name of tag */
-#define	TAG_WILD	4	/* tag pattern has wildcards */
+#define	TAG_REGEXP	4	/* use tag pattern as regexp */
+#define	TAG_NOIC	8	/* don't always ignore case */
 
 /*
  * Events for autocommands.
@@ -513,6 +516,7 @@ enum hlf_value
     HLF_N,	    /* line number for ":number" and ":#" commands */
     HLF_R,	    /* return to continue message and yes/no questions */
     HLF_S,	    /* status lines */
+    HLF_SNC,	    /* status lines of not-current windows */
     HLF_T,	    /* Titles for output from ":set all", ":autocmd" etc. */
     HLF_V,	    /* Visual mode */
     HLF_W,	    /* warning messages */
@@ -577,7 +581,7 @@ enum hlf_value
 #define IOSIZE	   (1024+1)	/* file i/o and sprintf buffer size */
 #define MSG_BUF_LEN 80		/* length of buffer for small messages */
 
-#if defined(AMIGA) || defined(__linux__) || defined(__QNX__) || defined(__CYGWIN32__)
+#if defined(AMIGA) || defined(__linux__) || defined(__QNX__) || defined(__CYGWIN32__) || defined(_AIX)
 # define TBUFSZ 2048		/* buffer size for termcap entry */
 #else
 # define TBUFSZ 1024		/* buffer size for termcap entry */
@@ -847,7 +851,7 @@ typedef struct VimClipboard
  * functions of these names. The declarations would break if the defines had
  * been seen at that stage.
  */
-#ifndef USE_GUI_WIN32
+#if !defined(USE_GUI_WIN32) && !defined(macintosh)
 # define mch_errmsg(str)	fprintf(stderr, (str))
 # define mch_display_error()	fflush(stderr)
 #endif
